(ns com.aurellem.gb.items
  (:use (com.aurellem.gb gb-driver util))
  (:import [com.aurellem.gb.gb_driver SaveState]))

(defn game-name
  "return pokemon-yellow header information"
  []
  (map char (subvec (vec (memory)) 0x134 0x142)))

(def item-list-start 0xD31C)

(defn item-list
  ([^SaveState state]
     (subvec
      (vec (memory state))
      item-list-start
      (+ item-list-start 255)))
  ([] (item-list @current-state)))
  
(def item-code->item-name
  (hash-map
   0x01 :master-ball       
   0x02 :ultra-ball
   0x03 :great-ball
   0x04 :poke-ball
   0x05 :town-map
   0x06 :bicycle
   0x07 :surfboard
   0x08 :safari-ball
   0x09 :pokedex
   0x0A :moon-stone
   0x0B :antidote
   0x0C :burn-heal
   0x0D :ice-heal
   0x0E :awakening
   0x0F :parlyz-heal
   0x10 :full-restore
   0x11 :max-potion
   0x12 :hyper-potion
   0x13 :super-potion
   0x14 :potion
   0x15 :boulderbadge
   0x16 :cascadebadge
   0x17 :thunderbadge
   0x18 :rainbowbadge
   0x19 :soulbadge
   0x1A :marshbadge
   0x1B :volcanobadge
   0x1C :earthbadge
   0x1D :escape-rope
   0x1E :repel
   0x1F :old-amber
   0x20 :fire-stone
   0x21 :thunderstone
   0x22 :water-stone
   0x23 :hp-up
   0x24 :protein
   0x25 :iron
   0x26 :carbos
   0x27 :calcium
   0x28 :rare-candy
   0x29 :dome-fossil
   0x2A :helix-fossil
   0x2B :secret-key
   0x2D :bike-voucher
   0x2E :x-accuracy
   0x2F :leaf-stone
   0x30 :card-key
   0x31 :nugget
   0x32 :pp-up-glitched
   0x33 :poke-doll
   0x34 :full-heal
   0x35 :revive
   0x36 :max-revive
   0x37 :guard-spec
   0x38 :super-repel
   0x39 :max-repel
   0x3A :dire-hit
   0x3B :coin
   0x3C :fresh-water
   0x3D :soda-pop
   0x3E :lemonade
   0x3F :s.s.ticket
   0x40 :gold-teeth
   0x41 :x-attack
   0x42 :x-defend
   0x43 :x-speed
   0x44 :x-special
   0x45 :coin-case
   0x46 :oaks-parcel
   0x47 :itemfinder
   0x48 :silph-scope
   0x49 :poke-flute
   0x4A :lift-key
   0x4B :exp.all
   0x4C :old-rod
   0x4D :good-rod
   0x4E :super-rod
   0x4F :pp-up
   0x50 :ether
   0x51 :max-ether
   0x52 :elixer
   0x53 :max-elixer
   0xC4 :HM01     ;; cut		   
   0xC5 :HM02     ;; fly		   
   0xC6 :HM03     ;; surf		   
   0xC7 :HM04     ;; strength	   
   0xC8 :HM05     ;; flash	   
   0xC9 :TM01     ;; mega punch	   
   0xCA :TM02     ;; razor wind	   
   0xCB :TM03     ;; swords dance	   
   0xCC :TM04     ;; whirlwind	   
   0xCD :TM05     ;; mega kick	   
   0xCE :TM06     ;; toxic	   
   0xCF :TM07     ;; horn drill	   
   0xD0 :TM08     ;; body slam	   
   0xD1 :TM09     ;; take down	   
   0xD2 :TM10     ;; double-edge	   
   0xD3 :TM11     ;; bubblebeam	   
   0xD4 :TM12     ;; water gun	   
   0xD5 :TM13     ;; ice beam 	   
   0xD6 :TM14     ;; blizzard	   
   0xD7 :TM15     ;; hyper beam	   
   0xD8 :TM16     ;; pay day	   
   0xD9 :TM17     ;; submission	   
   0xDA :TM18     ;; counter	   
   0xDB :TM19     ;; seismic toss	   
   0xDC :TM20     ;; rage		   
   0xDD :TM21     ;; mega drain	   
   0xDE :TM22     ;; solarbeam	   
   0xDF :TM23     ;; dragon rage	   
   0xE0 :TM24     ;; thunderbolt	   
   0xE1 :TM25     ;; thunder	   
   0xE2 :TM26     ;; earthquake	   
   0xE3 :TM27     ;; fissure	   
   0xE4 :TM28     ;; dig		   
   0xE5 :TM29     ;; psychic 	   
   0xE6 :TM30     ;; teleport	   
   0xE7 :TM31     ;; mimic	   
   0xE8 :TM32     ;; double team	   
   0xE9 :TM33     ;; reflect	   
   0xEA :TM34     ;; bide		   
   0xEB :TM35     ;; metronome	   
   0xEC :TM36     ;; self destruct   
   0xED :TM37     ;; egg bomb	   
   0xEE :TM38     ;; fire blast	   
   0xEF :TM39     ;; swift	   
   0xF0 :TM40     ;; skull bash	   
   0xF1 :TM41     ;; softboiled	   
   0xF2 :TM42     ;; dream eater	   
   0xF3 :TM43     ;; sky attack	   
   0xF4 :TM44     ;; rest		   
   0xF5 :TM45     ;; thunder wave	   
   0xF6 :TM46     ;; psywave	   
   0xF7 :TM47     ;; explosion	   
   0xF8 :TM48     ;; rock slide	   
   0xF9 :TM49     ;; tri attack	   
   0xFA :TM50     ;; substitute	   
   0xFB :TM51     ;; "cut"	   
   0xFC :TM52     ;; "fly"	   
   0xFD :TM53     ;; "surf"	   
   0xFE :TM54     ;; "strength"      
   0xFF :end-of-list-sentinel ;; also "flash"
   ))

(def item-name->item-code
  (zipmap (vals item-code->item-name)
          (keys item-code->item-name)))

(defn raw-inventory->inventory
  [raw-inventory]
  (map
     (fn [[item-code quantity]]
       [(item-code->item-name
         item-code
         item-code)
           quantity])
     (partition
      2
      raw-inventory)))

(defn inventory [^SaveState state]
  (let [items (item-list state)]
    (raw-inventory->inventory
     (next (take-while (partial not= 255) items)))))

(defn print-inventory
  ([] (print-inventory @current-state))
  ([^SaveState state]
     (println
      (let [inv (inventory state)]
        (reduce
         str
         (concat
          ["+-------------------+----------+\n"
           "|##| Item           | Quantity |\n"
           "+--+----------------+----------+\n"]

          (map
           (fn [index [item-name quantity]]
             (str 
              (format "|%-2d| %-14s | %3d      |\n" index
                      (apply str (rest (str item-name)))
                      quantity)))
           (range 0 (count inv)) inv)
          ["+--+----------------+----------+\n"]))))
     state))

(defn inventory-codes [inventory]
  (flatten
   (concat [(count inventory)]
           (map (fn [[item-name quantity]]
                  [(item-name->item-code item-name item-name)
                   quantity]) inventory)
           [(item-name->item-code :end-of-list-sentinel)])))

(defn set-inv-mem [^SaveState state inv-codes]
  (set-memory-range state item-list-start
                    inv-codes))
  

(defn set-inventory [^SaveState state new-inventory]
  (set-inv-mem state (inventory-codes new-inventory)))

(defn give-items
  ([^SaveState state items]
     (set-inventory state
                    (concat items (inventory state))))
  ([items]
     (give-items @current-state items)))

(defn clear-inventory
  ([^SaveState state]
     (set-inventory state []))
  ([] (clear-inventory @current-state)))


